#pragma once

#include "checks.h"
#include "config.h"
#include "timer_config.h"

/** \file Selects timer configuration based on user configuration in config.h.
 *  Defines general timer configuration macros without the index of timer to use. */

#if ATMOS_TIMER_INDEX == 0
#	define ATMOS_TIMER_BITS ATMOS_TIMER0_BITS
#	define ATMOS_TIMER_PRESCALER_CONTROL ATMOS_TIMER0_PRESCALER_CONTROL
#	define ATMOS_TIMER_PRESCALER_CS0 CS00
#	define ATMOS_TIMER_PRESCALER_CS1 CS01
#	define ATMOS_TIMER_PRESCALER_CS2 CS02
#	define ATMOS_TIMER_PRESCALER_CS3 CS03
#	ifdef ATMOS_TIMER0_MODE_CONTROL
#		define ATMOS_TIMER_MODE_CONTROL ATMOS_TIMER0_MODE_CONTROL
#		define ATMOS_TIMER_CTC_MODE_BIT ATMOS_TIMER0_CTC_MODE_BIT
#	endif //ATMOS_TIMER0_MODE_CONTROL
#	define ATMOS_TIMER_INTERRUPT_CONTROL ATMOS_TIMER0_INTERRUPT_CONTROL
#	define ATMOS_TIMER_COMPARE_INTERRUPT_BIT ATMOS_TIMER0_COMPARE_INTERRUPT_BIT
#	define ATMOS_TIMER_COMPARE_REGISTER ATMOS_TIMER0_COMPARE_REGISTER
#	define ATMOS_TIMER_INTERRUPT_NAME ATMOS_TIMER0_INTERRUPT_NAME
#	define ATMOS_TIMER_COUNTER ATMOS_TIMER0_COUNTER
#	ifdef ATMOS_TIMER0_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER0_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER0_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER0_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER0_PRESCALER_HAS_16384
#		define ATMOS_TIMER0_PRESCALER_HAS_16384
#	endif //ATMOS_TIMER0_PRESCALER_HAS_16384
#	ifdef ATMOS_TIMER0_HAS_16BIT_MODE
#		define ATMOS_TIMER_HAS_16BIT_MODE
#		define ATMOS_TIMER_16BIT_MODE_CONTROL ATMOS_TIMER0_16BIT_MODE_CONTROL
#		define ATMOS_TIMER_16BIT_MODE_BIT ATMOS_TIMER0_16BIT_MODE_BIT
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER ATMOS_TIMER0_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER ATMOS_TIMER0_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER
#	endif //ATMOS_TIMER0_HAS_16BIT_MODE
#elif ATMOS_TIMER_INDEX == 1
#	define ATMOS_TIMER_BITS ATMOS_TIMER1_BITS
#	define ATMOS_TIMER_PRESCALER_CONTROL ATMOS_TIMER1_PRESCALER_CONTROL
#	define ATMOS_TIMER_PRESCALER_CS0 CS10
#	define ATMOS_TIMER_PRESCALER_CS1 CS11
#	define ATMOS_TIMER_PRESCALER_CS2 CS12
#	define ATMOS_TIMER_PRESCALER_CS3 CS13
#	ifdef ATMOS_TIMER1_MODE_CONTROL
#		define ATMOS_TIMER_MODE_CONTROL ATMOS_TIMER1_MODE_CONTROL
#		define ATMOS_TIMER_CTC_MODE_BIT ATMOS_TIMER1_CTC_MODE_BIT
#	endif //ATMOS_TIMER1_MODE_CONTROL
#	define ATMOS_TIMER_INTERRUPT_CONTROL ATMOS_TIMER1_INTERRUPT_CONTROL
#	define ATMOS_TIMER_COMPARE_INTERRUPT_BIT ATMOS_TIMER1_COMPARE_INTERRUPT_BIT
#	define ATMOS_TIMER_COMPARE_REGISTER ATMOS_TIMER1_COMPARE_REGISTER
#	define ATMOS_TIMER_INTERRUPT_NAME ATMOS_TIMER1_INTERRUPT_NAME
#	define ATMOS_TIMER_COUNTER ATMOS_TIMER1_COUNTER
#	ifdef ATMOS_TIMER1_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER1_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER1_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER1_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER1_PRESCALER_HAS_16384
#		define ATMOS_TIMER1_PRESCALER_HAS_16384
#	endif //ATMOS_TIMER1_PRESCALER_HAS_16384
#	ifdef ATMOS_TIMER1_HAS_16BIT_MODE
#		define ATMOS_TIMER_HAS_16BIT_MODE
#		define ATMOS_TIMER_16BIT_MODE_CONTROL ATMOS_TIMER1_16BIT_MODE_CONTROL
#		define ATMOS_TIMER_16BIT_MODE_BIT ATMOS_TIMER1_16BIT_MODE_BIT
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER ATMOS_TIMER1_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER ATMOS_TIMER1_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER
#	endif //ATMOS_TIMER1_HAS_16BIT_MODE
#elif  ATMOS_TIMER_INDEX == 2
#	define ATMOS_TIMER_BITS ATMOS_TIMER2_BITS
#	define ATMOS_TIMER_PRESCALER_CONTROL ATMOS_TIMER2_PRESCALER_CONTROL
#	define ATMOS_TIMER_PRESCALER_CS0 CS20
#	define ATMOS_TIMER_PRESCALER_CS1 CS21
#	define ATMOS_TIMER_PRESCALER_CS2 CS22
#	define ATMOS_TIMER_PRESCALER_CS3 CS23
#	ifdef ATMOS_TIMER2_MODE_CONTROL
#		define ATMOS_TIMER_MODE_CONTROL ATMOS_TIMER2_MODE_CONTROL
#		define ATMOS_TIMER_CTC_MODE_BIT ATMOS_TIMER2_CTC_MODE_BIT
#	endif //ATMOS_TIMER2_MODE_CONTROL
#	define ATMOS_TIMER_INTERRUPT_CONTROL ATMOS_TIMER2_INTERRUPT_CONTROL
#	define ATMOS_TIMER_COMPARE_INTERRUPT_BIT ATMOS_TIMER2_COMPARE_INTERRUPT_BIT
#	define ATMOS_TIMER_COMPARE_REGISTER ATMOS_TIMER2_COMPARE_REGISTER
#	define ATMOS_TIMER_INTERRUPT_NAME ATMOS_TIMER2_INTERRUPT_NAME
#	define ATMOS_TIMER_COUNTER ATMOS_TIMER2_COUNTER
#	ifdef ATMOS_TIMER2_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER2_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER2_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER2_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER2_PRESCALER_HAS_16384
#		define ATMOS_TIMER2_PRESCALER_HAS_16384
#	endif //ATMOS_TIMER2_PRESCALER_HAS_16384
#	ifdef ATMOS_TIMER2_HAS_16BIT_MODE
#		define ATMOS_TIMER_HAS_16BIT_MODE
#		define ATMOS_TIMER_16BIT_MODE_CONTROL ATMOS_TIMER2_16BIT_MODE_CONTROL
#		define ATMOS_TIMER_16BIT_MODE_BIT ATMOS_TIMER2_16BIT_MODE_BIT
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER ATMOS_TIMER2_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER ATMOS_TIMER2_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER
#	endif //ATMOS_TIMER2_HAS_16BIT_MODE
#elif  ATMOS_TIMER_INDEX == 3
#	define ATMOS_TIMER_BITS ATMOS_TIMER3_BITS
#	define ATMOS_TIMER_PRESCALER_CONTROL ATMOS_TIMER3_PRESCALER_CONTROL
#	define ATMOS_TIMER_PRESCALER_CS0 CS30
#	define ATMOS_TIMER_PRESCALER_CS1 CS31
#	define ATMOS_TIMER_PRESCALER_CS2 CS32
#	define ATMOS_TIMER_PRESCALER_CS3 CS33
#	ifdef ATMOS_TIMER3_MODE_CONTROL
#		define ATMOS_TIMER_MODE_CONTROL ATMOS_TIMER3_MODE_CONTROL
#		define ATMOS_TIMER_CTC_MODE_BIT ATMOS_TIMER3_CTC_MODE_BIT
#	endif //ATMOS_TIMER3_MODE_CONTROL
#	define ATMOS_TIMER_INTERRUPT_CONTROL ATMOS_TIMER3_INTERRUPT_CONTROL
#	define ATMOS_TIMER_COMPARE_INTERRUPT_BIT ATMOS_TIMER3_COMPARE_INTERRUPT_BIT
#	define ATMOS_TIMER_COMPARE_REGISTER ATMOS_TIMER3_COMPARE_REGISTER
#	define ATMOS_TIMER_INTERRUPT_NAME ATMOS_TIMER3_INTERRUPT_NAME
#	define ATMOS_TIMER_COUNTER ATMOS_TIMER3_COUNTER
#	ifdef ATMOS_TIMER3_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER3_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER3_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER3_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER3_PRESCALER_HAS_16384
#		define ATMOS_TIMER3_PRESCALER_HAS_16384
#	endif //ATMOS_TIMER3_PRESCALER_HAS_16384
#	ifdef ATMOS_TIMER3_HAS_16BIT_MODE
#		define ATMOS_TIMER_HAS_16BIT_MODE
#		define ATMOS_TIMER_16BIT_MODE_CONTROL ATMOS_TIMER3_16BIT_MODE_CONTROL
#		define ATMOS_TIMER_16BIT_MODE_BIT ATMOS_TIMER3_16BIT_MODE_BIT
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER ATMOS_TIMER3_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER ATMOS_TIMER3_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER
#	endif //ATMOS_TIMER3_HAS_16BIT_MODE
#elif  ATMOS_TIMER_INDEX == 4
#	define ATMOS_TIMER_BITS ATMOS_TIMER4_BITS
#	define ATMOS_TIMER_PRESCALER_CONTROL ATMOS_TIMER4_PRESCALER_CONTROL
#	define ATMOS_TIMER_PRESCALER_CS0 CS40
#	define ATMOS_TIMER_PRESCALER_CS1 CS41
#	define ATMOS_TIMER_PRESCALER_CS2 CS42
#	define ATMOS_TIMER_PRESCALER_CS3 CS43
#	ifdef ATMOS_TIMER4_MODE_CONTROL
#		define ATMOS_TIMER_MODE_CONTROL ATMOS_TIMER4_MODE_CONTROL
#		define ATMOS_TIMER_CTC_MODE_BIT ATMOS_TIMER4_CTC_MODE_BIT
#	endif //ATMOS_TIMER4_MODE_CONTROL
#	define ATMOS_TIMER_INTERRUPT_CONTROL ATMOS_TIMER4_INTERRUPT_CONTROL
#	define ATMOS_TIMER_COMPARE_INTERRUPT_BIT ATMOS_TIMER4_COMPARE_INTERRUPT_BIT
#	define ATMOS_TIMER_COMPARE_REGISTER ATMOS_TIMER4_COMPARE_REGISTER
#	define ATMOS_TIMER_INTERRUPT_NAME ATMOS_TIMER4_INTERRUPT_NAME
#	define ATMOS_TIMER_COUNTER ATMOS_TIMER4_COUNTER
#	ifdef ATMOS_TIMER4_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER4_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER4_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER4_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER4_PRESCALER_HAS_16384
#		define ATMOS_TIMER4_PRESCALER_HAS_16384
#	endif //ATMOS_TIMER4_PRESCALER_HAS_16384
#	ifdef ATMOS_TIMER4_HAS_16BIT_MODE
#		define ATMOS_TIMER_HAS_16BIT_MODE
#		define ATMOS_TIMER_16BIT_MODE_CONTROL ATMOS_TIMER4_16BIT_MODE_CONTROL
#		define ATMOS_TIMER_16BIT_MODE_BIT ATMOS_TIMER4_16BIT_MODE_BIT
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER ATMOS_TIMER4_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER ATMOS_TIMER4_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER
#	endif //ATMOS_TIMER4_HAS_16BIT_MODE
#elif ATMOS_TIMER_INDEX == 5
#	define ATMOS_TIMER_BITS ATMOS_TIMER5_BITS
#	define ATMOS_TIMER_PRESCALER_CONTROL ATMOS_TIMER5_PRESCALER_CONTROL
#	define ATMOS_TIMER_PRESCALER_CS0 CS50
#	define ATMOS_TIMER_PRESCALER_CS1 CS51
#	define ATMOS_TIMER_PRESCALER_CS2 CS52
#	define ATMOS_TIMER_PRESCALER_CS3 CS53
#	ifdef ATMOS_TIMER5_MODE_CONTROL
#		define ATMOS_TIMER_MODE_CONTROL ATMOS_TIMER5_MODE_CONTROL
#		define ATMOS_TIMER_CTC_MODE_BIT ATMOS_TIMER5_CTC_MODE_BIT
#	endif //ATMOS_TIMER5_MODE_CONTROL
#	define ATMOS_TIMER_INTERRUPT_CONTROL ATMOS_TIMER5_INTERRUPT_CONTROL
#	define ATMOS_TIMER_COMPARE_INTERRUPT_BIT ATMOS_TIMER5_COMPARE_INTERRUPT_BIT
#	define ATMOS_TIMER_COMPARE_REGISTER ATMOS_TIMER5_COMPARE_REGISTER
#	define ATMOS_TIMER_INTERRUPT_NAME ATMOS_TIMER5_INTERRUPT_NAME
#	define ATMOS_TIMER_COUNTER ATMOS_TIMER5_COUNTER
#	ifdef ATMOS_TIMER5_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER5_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER5_PRESCALER_HAS_32_128
#		#define ATMOS_TIMER_PRESCALER_HAS_32_128
#	endif //ATMOS_TIMER5_PRESCALER_HAS_32_128
#	ifdef ATMOS_TIMER5_PRESCALER_HAS_16384
#		define ATMOS_TIMER5_PRESCALER_HAS_16384
#	endif //ATMOS_TIMER5_PRESCALER_HAS_16384
#	ifdef ATMOS_TIMER5_HAS_16BIT_MODE
#		define ATMOS_TIMER_HAS_16BIT_MODE
#		define ATMOS_TIMER_16BIT_MODE_CONTROL ATMOS_TIMER5_16BIT_MODE_CONTROL
#		define ATMOS_TIMER_16BIT_MODE_BIT ATMOS_TIMER5_16BIT_MODE_BIT
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER ATMOS_TIMER5_16BIT_MODE_HIGH_BYTE_COMPARE_REGISTER
#		define ATMOS_TIMER_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER ATMOS_TIMER5_16BIT_MODE_HIGH_BYTE_COUNTER_REGISTER
#	endif //ATMOS_TIMER5_HAS_16BIT_MODE
#else //ATMOS_TIMER_INDEX
static_assert(false, "ATMOS_TIMER_INDEX has unsupported value. Supported values are 0 to 5.");
#endif //ATMOS_TIMER_INDEX
